#!/usr/bin/perl


use utf8;
use Encode qw(decode encode decode_utf8 is_utf8);
use CGI qw(:standard);
use JSON::PP;
use File::Slurp;
use Data::Dumper;
use MIME::Base64;
use Digest::MD5 qw(md5 md5_hex md5_base64);

no warnings 'utf8';
#use open qw/ :std :encoding(utf8) /; ### This breaks binary file uploads (utf-8 issue on STDIN).

$CGI::POST_MAX = 1024 * 15000;


$prog = "admin.cgi";
$jsoneditor_html = "web_admin_docroot/jsoneditor.html";
$network_interfaces_template_file = "./interfaces.template";
$network_interfaces_file = "/var/lib/interfaces";
$evdevname = "/home/fabmicro/SIPHomePhone/evdevname";
$sunbus_version_file = "/tmp/sunbus.version.log";

$query_uri = $ENV{'REQUEST_URI'};

$query = new CGI;
$action = $query->param("action");


if(-f $ENV{X_MANAGED_CONFIG} ) {
	$current_config_text = read_file($ENV{X_MANAGED_CONFIG}, {binmode => ':utf8'});
	#$current_config_text = read_file($ENV{X_MANAGED_CONFIG});
} elsif(-f $ENV{X_DEFAULT_CONFIG}) {
	$current_config_text = read_file($ENV{X_DEFAULT_CONFIG}, {binmode => ':utf8'});
	#$current_config_text = read_file($ENV{X_DEFAULT_CONFIG});
} elsif(-f "all.json.default") {
	$current_config_text = read_file("all.json.default", {binmode => ':utf8'});
	#$current_config_text = read_file("all.json.default");
} else {
	print "Content-type: text/html; charset=utf-8\n\n";
	print "Cannot find JSON config!\n";
	exit(3);
}

# Build a char->hex map
my %escapes;
for (0..255) {
    $escapes{chr($_)} = sprintf("%%%02X", $_);
}
my %subst;  # compiled patterns
my %Unsafe = (
    RFC2732 => qr/[^A-Za-z0-9\-_.!~*'()]/,
    RFC3986 => qr/[^A-Za-z0-9\-\._~]/,
);


$enable = true;
$json_parser = JSON::PP->new;
$json_parser = $json_parser->relaxed($enable);
$json_parser = $json_parser->pretty($enable);
$json_parser = $json_parser->allow_barekey($enable);
$json_parser = $json_parser->canonical($enable);
#eval { $current_config_json  = $json_parser->utf8->decode($current_config_text); };
eval { $current_config_json  = $json_parser->decode($current_config_text); };

$loaded_config_serial = $current_config_json->{Config}->{serial};
$loaded_config_source = $current_config_json->{Config}->{source};

$EDITABLE_SECTIONS = {
	LockManager => {
		descr => 'Менеджер замков',
		type => 'editable',
		schema => 'web_admin_docroot/schema/lock_manager.json.schema',
	},
	HandsetState => {
		descr => 'Датчик аналоговой трубки',
		type => 'editable',
		schema => 'web_admin_docroot/schema/handset_state.json.schema',
	},	
	ProximitySensor => {
		descr => 'Датчик присутствия',
		type => 'editable',
		schema => 'web_admin_docroot/schema/proximity_sensor.json.schema',
	},
	AmbientSensor => {
		descr => 'Датчик освещенности',
		type => 'editable',
		schema => 'web_admin_docroot/schema/ambient_sensor.json.schema',
	},
	RS485Master => {
		descr => 'Мастер RS485 интерфейса',
		type => 'editable',
		schema => 'web_admin_docroot/schema/rs485_master.json.schema',
	},
	RFIDReader => {
		descr => 'Считыватель RFID меток',
		type => 'editable',
		schema => 'web_admin_docroot/schema/rfid_reader.json.schema',
	},
	DoorsAndButtons => {
		descr => 'Положение дверей и состояние кнопок',
		type => 'editable',
		schema => 'web_admin_docroot/schema/doors_and_buttons.json.schema',
	},
	Notify => {
		descr => 'Оповещения о событиях',
		type => 'editable',
		schema => 'web_admin_docroot/schema/notify.json.schema',
	},
	Common => {
		descr => 'Настройки общие для разных модулей',
		type => 'editable',
		schema => 'web_admin_docroot/schema/common.json.schema',
	},
	Apartments => {
		descr => 'Помещения и RFID ключи',
		type => 'editable',
		schema => 'web_admin_docroot/schema/apartments.json.schema',
	},
	Yard => {
		descr => 'Двор',
		type => 'editable',
		schema => 'web_admin_docroot/schema/yard.json.schema',
	},
	Advertizer => {
		descr => 'Рекламные баннеры',
		type => 'editable',
		schema => 'web_admin_docroot/schema/advertizer.json.schema',
	},
	GUI => {
		descr => 'Графический интерфейс',
		type => 'editable',
		schema => 'web_admin_docroot/schema/gui.json.schema',
	},
};


if(length($action) < 1) {
	$action = $current_config_json->{WebAdmin}->{default_action}; 

	if(length($action) < 1) {
		$action = 'list_sections';
	}
}

if($action eq 'wizard') {

	header();

	print "Мастер настройки:\n";
	print "<HR>\n";
	print "<UL>\n";

	foreach $section (sort keys %$EDITABLE_SECTIONS) {
		if($EDITABLE_SECTIONS->{$section}->{type} eq 'editable') {
			print "<LI> <a href=\"$prog?action=edit_section&section=$section\">$section - ".$EDITABLE_SECTIONS->{$section}->{descr}."</a>\n";
		} elsif ($EDITABLE_SECTIONS->{$section}->{type} eq 'visible') {
			print "<LI> $section - ".$EDITABLE_SECTIONS->{$section}->{descr}."\n";
		}
	}
	print "</UL>\n";

	footer();
	exit(0);
}

if($action eq 'list_sections') {

	header();

	print "Конфигурационные разделы:\n";
	print "<HR>\n";
	print "<UL>\n";

	foreach $section (sort keys %$EDITABLE_SECTIONS) {
		if($EDITABLE_SECTIONS->{$section}->{type} eq 'editable') {
			print "<LI> <a href=\"$prog?action=edit_section&section=$section\">$section - ".$EDITABLE_SECTIONS->{$section}->{descr}."</a>\n";
		} elsif ($EDITABLE_SECTIONS->{$section}->{type} eq 'visible') {
			print "<LI> $section - ".$EDITABLE_SECTIONS->{$section}->{descr}."\n";
		}
	}
	print "</UL>\n";

	footer();
	exit(0);
}


if($action eq 'edit_section') {

	$section = $query->param("section");

	header();

	if(($EDITABLE_SECTIONS->{$section}->{type} eq 'editable')) {
		#print "Конфигурационный раздел: ".$EDITABLE_SECTIONS->{$section}->{descr}."\n";
		print "Конфигурационный раздел:\n";
		print "<HR>\n";

		if(! exists($current_config_json->{$section})) { $current_config_json->{$section} = (); }

		if(exists ($current_config_json->{$section})  && -f $EDITABLE_SECTIONS->{$section}->{schema}) {
			eval { $section_json_text  = $json_parser->utf8->encode($current_config_json->{$section}); };
			#eval { $section_json_text  = $json_parser->encode($current_config_json->{$section}); };
		
			$data_base64 = encode_base64($section_json_text);
			$data_base64 =~ s/\n/\"+\n\"/g;
			
			#$schema_base64 = encode_base64(read_file($EDITABLE_SECTIONS->{$section}->{schema}, {binmode => ':utf8'}));
			$schema_base64 = encode_base64(read_file($EDITABLE_SECTIONS->{$section}->{schema}));
			$schema_base64 =~ s/\n/\"+\n\"/g;

			$html = read_file($jsoneditor_html, {binmode => ':utf8'});

			$html =~ s/__SECTION_ID__/$section/g;
			$html =~ s/__SECTION_NAME__/$EDITABLE_SECTIONS->{$section}->{descr}/g;
			$html =~ s/__DATA__/$data_base64/g;
			$html =~ s/__SCHEMA__/$schema_base64/g;

			print $html;

		} else {
			print "Отсутствует JSON схема для данного раздела.\n";
		}
	} else { 
		print "Конфигурационный раздел: ".$EDITABLE_SECTIONS->{$section}->{descr}."\n";
		print "<HR>\n";
		if(exists ($current_config_json->{$section}) ) {
			print "<PRE>\n";
			eval { $section_json_text  = "\"".$section."\" : ".$json_parser->utf8->encode($current_config_json->{$section}); };
			print "$section_json_text\n";
			print "</PRE>\n";
		} else {
			print "В данный момент раздел отсутствует\n";
		}
	}


	footer();
	exit(0);
}


if($action eq 'system_settings') {

	header();

	print "Системные операции\n";
	print "<HR>\n";
	print "<UL>\n";
	print "<LI> <a href=\"$prog?action=system_view_config_spool_dir\">Просмотреть конфигурацинный спул</a>\n";
	print "<LI> <a href=\"$prog?action=system_view_all_json\">Просмотреть конфигурационный файл</a>\n";
	print "<LI> <a href=\"$prog?action=system_download_all_json\">Скачать конфигурационный файл</a>\n";
	print "<LI> <a href=\"$prog?action=system_upload_all_json\">Загрузить конфигурационный файл</a>\n";
	print "</UL>\n";
	print "<UL>\n";
	print "<LI> <a href=\"$prog?action=system_view_media_spool_dir\">Просмотреть медиа спул</a>\n";
	print "<LI> <a href=\"$prog?action=system_view_request_params\">Просмотреть параметры HTTP запроса</a>\n";
	print "<LI> <a href=\"$prog?action=system_device_diagnostics\">Диагностика устройств</a>\n";
	print "<LI> <a href=\"$prog?action=system_view_monitor\">Монитор системных событий</a>\n";
	print "<LI> <a href=\"$prog?action=system_test_call\">Инициировать звонок в помещение</a>\n";
	print "</UL>\n";
	print "<UL>\n";
	print "<LI> <a href=\"$prog?action=system_view_modules_version\">Просмотреть версии модулей</a>\n";
	print "<LI> <a href=\"$prog?action=system_upload_update\">Загрузить обновление ПО</a>\n";
	print "</UL>\n";
	print "<UL>\n";
	print "<LI> <a href=\"\$restart\$\">Перезапусить все модули</a>\n";
	print "<LI> <a href=\"\$reboot\$\">Перезагрузить систему</a>\n";
	print "<LI> <a href=\"\$poweroff\$\">Выключить питание</a>\n";
	print "</UL>\n";

	footer();
	exit(0);
}



if($action eq 'network_settings') {

	header();

	print "Сетевые настройки\n";
	print "<HR>\n";
	print "<UL>\n";
	print "<LI> <a href=\"$prog?action=system_view_network_interfaces\">Просмотреть сетевые настройки</a>\n";
	print "<LI> <a href=\"$prog?action=network_change_network_interfaces\">Установить IP адрес/маску/шлюз/DHCP</a>\n";
	print "<LI> <a href=\"$prog?action=system_view_dhcp_lease\">Просмотреть DHCP lease</a>\n";
	print "<LI> <a href=\"$prog?action=network_change_web_credentials\">Изменить имя пользователя и пароль для Web интерфейса</a>\n";
	print "<LI> <a href=\"$prog?action=network_change_config_credentials\">Изменить адрес сервера конфигурации и пароль к нему</a>\n";
	print "</UL>\n";

	footer();
	exit(0);
}


if($action eq 'network_change_web_credentials') {

	header();

	print "Изменить имя пользователя и пароль для Web интерфейса:\n";
	print "<HR>\n";
	print "<FORM method=\"POST\" action=\"$prog\">\n";
	print "<INPUT type=hidden name=action value=network_change_web_credentials_submit>\n";
	print "<TABLE>\n";
	print "<TR><TD>Текущее имя пользователя: </TD><TD><INPUT name=old_username type=text onChange='new_username.value = old_username.value;' value='".$current_config_json->{WebAdmin}->{web_admin}."'></TD></TR>\n";
	print "<TR><TD>Текущий пароль: </TD><TD><INPUT name=old_password type=password value=''></TD></TR>\n";
	print "<TR><TD>Новое имя пользователя: </TD><TD><INPUT name=new_username type=text></TD></TR>\n";
	print "<TR><TD>Новый пароль: </TD><TD><INPUT name=new_password type=password></TD></TR>\n";
	print "<TR><TD colspan=2><INPUT type=submit value='Сохранить' style='width:100%'></TD></TR>\n";
	print "</TABLE>\n";
	print "</FORM>\n";

	footer();
	exit(0);
}


if($action eq 'network_change_web_credentials_submit') {

	$old_username = $query->param("old_username");
	$old_password = $query->param("old_password");
	$new_username = $query->param("new_username");
	$new_password = $query->param("new_password");

	header();

	print "Изменить имя пользователя и пароль для Web интерфейса:\n";
	print "<HR>\n";

	if($old_username eq $current_config_json->{WebAdmin}->{web_admin} &&
	   $old_password eq $current_config_json->{WebAdmin}->{web_password} ) {

		$current_config_json->{WebAdmin}->{web_admin} = $new_username;
		$current_config_json->{WebAdmin}->{web_password} = $new_password;

		save_config();

		print "Изменения сохранены и примут силу после <a href='\$restart\$'>перезапуска всех модулей</a> или <a href='\$reboot\$'>перезагрузки системы</a>.\n";
	} else {
		print "Текущие имя пользователя или пароль не совпадают!\n";
	}

	footer();
	exit(0);
}


if($action eq 'network_change_config_credentials') {

	header();

	print "Изменить адрес сервера конфигурации и пароль к нему:\n";
	print "<HR>\n";
	print "<FORM method=\"POST\" action=\"$prog\">\n";
	print "<INPUT type=hidden name=action value=network_change_config_credentials_submit>\n";
	print "<TABLE>\n";
	print "<TR><TD>URL сервера: </TD><TD><INPUT name=config_url type=text value='".$current_config_json->{Config}->{config_url}."'> (ftp://, tftp://, http:// или https://)</TD></TR>\n";
	print "<TR><TD>Новый пароль: </TD><TD><INPUT name=new_password type=password></TD></TR>\n";
	print "<TR><TD colspan=2><INPUT type=submit value='Сохранить' style='width:100%'></TD></TR>\n";
	print "</TABLE>\n";
	print "</FORM>\n";

	footer();
	exit(0);
}



if($action eq 'network_change_config_credentials_submit') {

	$config_url = $query->param("config_url");
	$new_password = $query->param("new_password");

	header();

	print "Изменить адрес сервера конфигурации и пароль к нему:\n";
	print "<HR>\n";

	$current_config_json->{Config}->{config_url} = $config_url;

	save_config();

	$new_password .= "\n";
	$config_password_file = $ENV{X_CONFIG_SPOOL_DIR} . "/" . $current_config_json->{Configurator}->{password_file_name}; 

	write_file($config_password_file, $new_password);


	print "Изменения сохранены и примут силу после <a href='\$restart\$'>перезапуска всех модулей</a> или <a href='\$reboot\$'>перезагрузки системы</a>.\n";

	footer();
	exit(0);
}


if($action eq 'network_change_network_interfaces') {

	header();


	open(IFCONFIG, "/sbin/ifconfig eth0|");
	while(<IFCONFIG>) { if (/ inet /) { $ifconfig = $_; last; }}
	close(IFCONFIG);
	
	$IP = $ifconfig; $IP =~ s/.*inet ([0-9]+\.[0-9]+.[0-9]+.[0-9]+) .*/\1/g;
	$MASK = $ifconfig; $MASK =~ s/.*netmask ([0-9]+\.[0-9]+.[0-9]+.[0-9]+) .*/\1/g;

	open(NETSTAT, "/bin/netstat -rn|");
	while(<NETSTAT>) { if (/^0.0.0.0/) { ($_, $GATEWAY, @_) = split(' ', $_); last; }}



	print "Установить IP адрес/маску/шлюз/DHCP:\n";
	print "<HR>\n";
	print "<FORM method=\"POST\" action=\"$prog\">\n";
	print "<INPUT type=hidden name=action value=network_change_network_interfaces_submit>\n";
	print "<TABLE>\n";
	print "<TR><TD>Типа: </TD><TD><SELECT name=type><OPTION value=dhcp>DHCP</OPTION><OPTION value=static>STATIC</OPTION></SELECT></TD></TR>\n";
	print "<TR><TD>IP: </TD><TD><INPUT type=text name=ip value=\"$IP\"></TD></TR>\n";
	print "<TR><TD>Маска: </TD><TD><INPUT type=text name=mask value=\"$MASK\"></TD></TR>\n";
	print "<TR><TD>Шлюз: </TD><TD><INPUT type=text name=gateway value=\"$GATEWAY\"></TD></TR>\n";
	print "<TR><TD colspan=2><INPUT type=submit value='Сохранить' style='width:100%'></TD></TR>\n";
	print "</TABLE>\n";
	print "</FORM>\n";

	footer();
	exit(0);
}


if($action eq 'network_change_network_interfaces_submit') {

	$ip = $query->param("ip");
	$mask = $query->param("mask");
	$gateway = $query->param("gateway");
	$type = $query->param("type");

	$ip =~ s/ //g;
	$mask =~ s/ //g;
	$gateway =~ s/ //g;

	header();

	print "Установить IP адрес/маску/шлюз/DHCP:\n";
	print "<HR>\n";

	if($ip !~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]/) {
		print "Неверно задан IP: $ip\n";
	} elsif($mask !~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]/) {
		print "Неверно задана маска: $mask\n";
	} elsif($gateway !~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]/) {
		print "Неверно задан шлюз: $gateway\n";
	} elsif(!($type eq 'static') && !($type eq 'dhcp')) {
		print "Неверно задан тип интерфейса: $type\n";
	} else {
		print "<TABLE>\n";
		print "<TR><TD>Тип интерфейса: </TD><TD>$type</TD></TR>\n";
		print "<TR><TD>IP адрес: </TD><TD>$ip</TD></TR>\n";
		print "<TR><TD>Маска: </TD><TD>$mask</TD></TR>\n";
		print "<TR><TD>Шлюз: </TD><TD>$gateway</TD></TR>\n"; 
		print "</TABLE><BR>\n";

		print "Изменения сохранены и примут силу после <a href='\$reboot\$'>перезагрузки системы</a>.\n";

		$network_interfaces = read_file($network_interfaces_template_file);
		$network_interfaces =~ s/__TYPE__/$type/g;
		$network_interfaces =~ s/__IP__/$ip/g;
		$network_interfaces =~ s/__MASK__/$mask/g;
		$network_interfaces =~ s/__GATEWAY__/$gateway/g;

		write_file($network_interfaces_file, $network_interfaces);
	}

	footer();
	exit(0);

}

if($action eq 'system_download_all_json') {


	print "Content-type: application/json; charset=utf-8\nContent-Disposition: form-data; name=\"all.json\"; filename=\"all.json\"\n\n";
	print $current_config_text."\n";

	exit(0);
}

if($action eq 'system_upload_all_json') {

	header();

	print "Загрузить конфигурационный файл:\n";
	print "<HR>\n";
	print "<FORM method=\"POST\" action=\"$prog\" enctype='multipart/form-data'>\n";
	print "<INPUT type=hidden name=action value=system_upload_all_json_submit>\n";
	print "<TABLE>\n";
	print "<TR><TD colspan=2><INPUT name=file type=file accept='application/json, text/plain'></TD></TR>\n";
	print "<TR><TD colspan=2><INPUT type=submit value='Загрузить' style='width:100%'></TD></TR>\n";
	print "</TABLE>\n";
	print "</FORM>\n";

	footer();
	exit(0);
}


if($action eq 'system_upload_all_json_submit') {

	$file_handle = $query->upload("file");
	binmode($file_handle, ":utf8");

	my $loaded_config_text;

	while(<$file_handle>) {
		$loaded_config_text .= $_;
	}

	header();

	print "Загруженный конфигурационный файл:\n";
	print "<HR>\n";

	$current_config_text = $loaded_config_text;

	save_config();

	print "<PRE>\n";
	print "$loaded_config_text\n";
	print "</PRE>\n";

	footer();


	exit(0);
}



if($action eq 'system_view_all_json') {

	header();

	print "Текущий конфигурационный файл:\n";
	print "<HR>\n";
	print "<PRE>\n";
	print "$current_config_text\n";
	print "</PRE>\n";

	footer();
	exit(0);
}


if($action eq 'system_view_request_params') {

	header();

	print "Параметры HTTP запроса:\n";
	print "<HR>\n";
	print "<PRE>\n";
	system("env");
	print "</PRE>\n";

	footer();
	exit(0);
}


if($action eq 'system_view_config_spool_dir') {

	header();

	print "Конфигурационный спул:\n";
	print "<HR>\n";
	print "<FORM><PRE>\n";
	open(LS, "/bin/ls -l ".$ENV{X_CONFIG_SPOOL_DIR} ."|");
	while(<LS>) {
		chop;
		($a1, $a2, $a3, $a4, $a5, $a6, $a7, $a8, $a9, $a10) = split (' ', $_);
		if(length($a9) < 1) { next; }
		print "<INPUT type=button value=\"X\" hint=\"Удалить\" onClick=\"if(confirm('Удалить файл: $a9 ?')){self.location='$prog?action=system_delete_spool_file&name=$a9'}\"><INPUT type=button value=\"V\" hint=\"Просмотреть\" onClick=\"self.location='$prog?action=system_view_spool_file&name=$a9'\"><INPUT type=button value=\"D\" hint=\"Скачать\" onClick=\"self.location='$prog?action=system_download_spool_file&name=$a9'\"> $_ \n";
	}
	close(LS);
	print "</PRE></FORM>\n";

	footer();
	exit(0);
}

if($action eq 'system_view_media_spool_dir') {

	header();

	my $media_dir = $current_config_json->{HTTPRequester}->{media_dir};

	print "Медиа спул:\n";
	print "<HR>\n";
	print "<FORM><PRE>\n";
	open(LS, "/bin/ls -l ". $media_dir ."|");
	while(<LS>) {
		chop;
		($a1, $a2, $a3, $a4, $a5, $a6, $a7, $a8, $a9, $a10) = split (' ', $_);
		if(length($a9) < 1) { next; }
		print "<INPUT type=button value=\"X\" onClick=\"if(confirm('Удалить файл: $a9 ?')){self.location='$prog?action=system_delete_media_spool_file&name=$a9'}\"><INPUT type=button value=\"V\" onClick=\"self.location='$prog?action=system_view_media_spool_file&name=$a9'\"> $_ \n";
	}
	close(LS);
	print "</PRE></FORM>\n";

	footer();
	exit(0);
}


if($action eq 'system_view_spool_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	$file = $ENV{X_CONFIG_SPOOL_DIR}."/".$name;

	header();

	print "Просмотр файла в конфигурационном спуле: $name\n";
	print "<HR>\n";
	print "<PRE>\n";

	if(-f $file) {
		$text = read_file($file , {binmode => ':utf8'});
		print "$text\n";
	} else {
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_view_config_spool_dir\";</script>\n";
	}

	print "</PRE>\n";

	footer();
	exit(0);
}


if($action eq 'system_download_spool_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	$file = $ENV{X_CONFIG_SPOOL_DIR}."/".$name;

	if(-f $file) {
		print "Content-type: application/json\n\n";
		$text = read_file($file , {binmode => ':utf8'});
		print "$text\n";
	} else {
		header();
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_view_config_spool_dir\";</script>\n";
		footer();
	}


	exit(0);
}


if($action eq 'system_view_media_spool_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	$file = $current_config_json->{HTTPRequester}->{media_dir}."/".$name;

	header();

	print "Просмотр файла медиа спуле: $name\n";
	print "<HR>\n";
	print "<PRE>\n";

	if(-f $file) {
		$text = read_file($file , {binmode => ':utf8'});
		print "$text\n";
	} else {
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_view_media_spool_dir\";</script>\n";
	}

	print "</PRE>\n";

	footer();
	exit(0);
}





if($action eq 'system_delete_spool_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	$file = $ENV{X_CONFIG_SPOOL_DIR}."/".$name;

	header();
	

	if(-f $file) {
		unlink $file;
		print "<script>alert(\"Файл $name удален!\");self.location=\"$prog?action=system_view_config_spool_dir\";</script>\n";
	} else {
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_view_config_spool_dir\";</script>\n";
	}

	footer();
	exit(0);
}


if($action eq 'system_delete_media_spool_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	$file = $current_config_json->{HTTPRequester}->{media_dir}."/".$name;

	header();
	

	if(-f $file) {
		unlink $file;
		print "<script>alert(\"Файл $name удален!\");self.location=\"$prog?action=system_view_media_spool_dir\";</script>\n";
	} else {
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_view_media_spool_dir\";</script>\n";
	}

	footer();
	exit(0);
}



if($action eq 'system_view_dhcp_lease') {

	header();

	print "Текущий DHCP lease файл:\n";
	print "<HR>\n";
	if(exists ($current_config_json->{Configurator}) && -f $current_config_json->{Configurator}->{dhcp_lease_file} ) {
		$dhcp_lease_text = read_file($current_config_json->{Configurator}->{dhcp_lease_file});
		print "<PRE>\n";
		print "$dhcp_lease_text\n";
		print "</PRE>\n";
	} else {
		print "Отсутствует информация о файле DHCP lease.\n";
	}

	footer();
	exit(0);
}



if($action eq 'system_view_network_interfaces') {

	header();

	print "Текущие сетевые настройки:\n";
	print "<HR>\n";
	print "<UL>\n";
	print "<LI> /etc/network/interfaces\n";
	print "<PRE>\n";
	system("cat /etc/network/interfaces");
	print "</PRE>\n";
	print "<LI> /etc/resolv.conf\n";
	print "<PRE>\n";
	system("cat /etc/resolv.conf");
	print "</PRE>\n";
	print "<LI> /sbin/ifconfig -a\n";
	print "<PRE>\n";
	system("/sbin/ifconfig -a");
	print "</PRE>\n";
	print "</UL>\n";

	footer();
	exit(0);
}


if($action eq 'system_device_diagnostics') {

	$| = 1;
	use IO::Select;

	header();
	print "Диагностика устройств:\n";
	print "<HR>\n";
	print "<PRE>\n";



	my $pid = open($fd, "/etc/diag.pl|");

	while(<$fd>) {

		$msg = $_;
		chop($msg);

		$msg =~ s/\[1;31m/<font color=red>/g;
		$msg =~ s/\[1;32m/<font color=green>/g;
		$msg =~ s/\[0m/<\/font>/g;

		print $msg."\r\n";
	}

	close($fd);

	footer();
	exit(0);
}


if($action eq 'system_test_call') {

	header();

	my $media_dir = $current_config_json->{HTTPRequester}->{media_dir};

	print "Инициировать звонок в помещение:\n";
	print "<HR>\n";
	print "<FORM><PRE>\n";

	print "<TABLE border=1>\n";

	print "<TR align=center><TD> <B>Номер помещения:</B> </TD><TD> <B>Наименование пом. (собственник):</B> </TD><TD> <B>Марштуры:</B> </TD></TR>\n";

	if(exists ($current_config_json->{'Apartments'})) {
		foreach $apt_json_item (@{$current_config_json->{Apartments}}) {
			my $apt_num = $apt_json_item->{apt_num};
			my $descr = $apt_json_item->{descr};
			my $routes = $apt_json_item->{routes};
			my $routes_text = "";

			foreach $route (@{$routes}) {
				if($route->{type} eq direct) {
					$routes_text .= "Аналоговая трубка";
				}

				if($route->{type} eq switch) {
					$routes_text .= "Коммутатор D:E = ".$route->{D}.":".$route->{E};
				}

				if($route->{type} eq emergency) {
					$routes_text .= "ГОиЧС";
				}

				if($route->{type} eq sip) {
					$routes_text .= "SIP UriTo: ".$route->{UriTo};
				}

				$routes_text .= " ";
			}


			print "<TR><TD align=center><INPUT type=button value=\"$apt_num\" onClick=\"if(confirm('Инициировать звонок в помещение $apt_num ?')){window.open('$prog?action=system_test_call_init&apt_num=$apt_num');}\"></TD><TD> $descr </TD><TD> $routes_text </TD></TR>\n";
		}
	} else {
		print "<TR><TD colsan=3><B>Список помещений пуст</B></TD></TR>\n";
	}

	print "</TABLE>\n";
	print "</PRE></FORM>\n";

	footer();
	exit(0);
}


if($action eq 'save_section') {

	$section = $query->param("section");
	$content = $query->param("content");

	$content = decode('UTF-8',decode_base64($content));
	#$section_json = $json_parser->utf8->decode($content);
	#$section_text = $json_parser->utf8->encode($section_json);
	$section_json = $json_parser->decode($content);
	#$section_text = $json_parser->utf8->encode($section_json);
	$section_text = $json_parser->encode($section_json);


	header();

	if(($EDITABLE_SECTIONS->{$section}->{type} eq 'editable')) {
		print "Конфигурационный раздел: $section - ".$EDITABLE_SECTIONS->{$section}->{descr}."\n";
		print "<HR>\n";
		print "Получены и сохранены следующие данные:<BR><BR>\n\n";
		print "<PRE>\"$section\" : $section_text\n</PRE>";

		$current_config_json->{$section} = $section_json;

		if(save_config()) {
			print "<script>if(confirm('Перезагрузить модули ?')){self.location='\$restart\$';}</script>\n";
		} else {
			print "<PRE>\n";
			print "Возникла ошибка при сохранении конфигурационного файла!\n";
			print "1. Проверте наличие свободного пространства на накопителе в разделе <b>/var</b>\n";
			print "2. Удалите невостребованные конфигурационные файлы в <a href='$prog?action=system_view_config_spool_dir'>конфигурационном спуле</a>.\n";
			print "3. Удалите невостребованные медиа файлы в <a href='$prog?action=system_view_media_spool_dir'>медиа-спуле</a>.\n";
			print "</PRE>\n";
		}

	} else { 
		print "Конфигурационный раздел: ".$EDITABLE_SECTIONS->{$section}->{descr}." ($section) - ИЗМЕНЕНИЮ НЕ ПОДЛЕЖИТ\n";
		print "<HR>\n";
		if(exists ($current_config_json->{$section}) ) {
			print "<PRE>\n";
			eval { $section_json_text  = "\"".$section."\" : ".$json_parser->utf8->encode($current_config_json->{$section}); };
			print "$section_json_text\n";
			print "</PRE>\n";
		} else {
			print "В данный момент раздел отсутствует\n";
		}
	}

	footer();
	exit(0);
}


if($action eq 'sensors') {

	header();

	print "Показания датчиков:\n";
	print "<HR>\n";
	print "<UL>\n";
	print "<LI> <a href=\"$prog?action=proximity_sensor\">Датчик присутствия</a>\n";
	print "<LI> <a href=\"$prog?action=ambient_sensor\">Датчик освещенности</a>\n";
	print "<LI> <a href=\"$prog?action=handset_state\">Датчик напряжения в аналоговой линии</a>\n";
	print "<LI> <a href=\"$prog?action=doors_and_buttons_sensors\">Датчик положения дверей и кнопок</a>\n";
	print "<LI> <a href=\"$prog?action=rfid_reader\">Считыватель RFID меток/ключей</a>\n";
	print "</UL>\n";

	footer();
	exit(0);
}

if($action eq 'rfid_reader') {
	header();
        print "<script>self.location='$prog?filter=MFRC522_READER.*card_present&action=system_view_monitor'</script>\n";
        exit(0);
}

if($action eq 'rfid_reader_old') {

	$| = 1;
	use IO::Select;

	header();
	print "Считыватель RFID меток/ключей:\n";
	print "<HR>\n";
	print "<PRE>\n";

	my $dev = $current_config_json->{RFIDReader}->{serial};

	if(length($dev) < 1) {
		print "Файл-устройство не найден для устройства с именем '".$current_config_json->{RFIDReader}->{serial}."', проверьте правильность ввода имени устройства в конфигурационном файле, а так же наличие устройств с таким именем в разделе <a href=\"$prog?action=system_device_diagnostics\">Диагностика Устройств</a>\n";
		footer();
		exit(0);
	}

	# printf(STDERR "Opening device: $dev\n");

	my $fd_event = \*EVENT;
	my $fd_stdin = \*STDIN;

	my $pid = open($fd_event, "/bin/cat ".$dev."|");
	#my $pid = open($fd_event, $dev);

	if(!$pid) {
		print "Файл-устройство с именем $dev недоступно! Проверьте права доступа к файлу, его принадлежность к пользователю и эффективное имея пользователя для модуля WebAdmin - модуль должен иметь доступ к этому файлу!\n";
		footer();
		exit(0);
	}

	$s = IO::Select->new();
	$s->add($fd_stdin);
	$s->add($fd_event);

	my $terminate = 0;

	my $lines = 0;

	while($terminate == 0) {

		@fds = $s->has_exception(0);

		last if($fds > 0);

		@fds = $s->can_read(1);

		foreach my $fd (@fds) {
			if($fd == $fd_stdin) { $terminate = 1; next; }

			$line = <$fd>;
			chop($line);

			next if($line =~ /RAW DATA/);

			print time() . ": $line <script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
			STDOUT->flush();
		}

		$lines++;
	}

	kill(9, $pid);
	close($fd_event);

	footer();
	exit(0);
}


if($action eq 'proximity_sensor') {

	$| = 1;
	use IO::Select;
	$s = IO::Select->new();
	$s->add(\*STDIN);

	header();
	print "Датчик присутствия:\n";
	print "<HR>\n";

	my $device_file = $current_config_json->{ProximitySensor}->{hwmon_device};


	if(length($device_file) < 1) {
		print "Файл-устройство не найден для устройства с именем '".$current_config_json->{ProximitySensor}->{hwmon_device}."', проверьте правильность ввода имени устройства в конфигурационном файле, а так же наличие устройств с таким именем в разделе <a href=\"$prog?action=system_device_diagnostics\">Диагностика Устройств</a>\n";
		footer();
		exit(0);
	}

	while(1) {
		$! = 0;
		$ret = $s->can_read(.5);
		if($ret != 0) {
			last;
		}

		$range = read_file($device_file);
		chop($range);

		print time() . " - millimeters: $range<BR>\n";
		print "<script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
		STDOUT->flush();
	}

	print "BYE\n";

	footer();
	exit(0);

}


if($action eq 'ambient_sensor') {

	$| = 1;
	use IO::Select;
	$s = IO::Select->new();
	$s->add(\*STDIN);

	header();
	print "Датчик освещенности:\n";
	print "<HR>\n";

	my $device_file = FindHwmonDevice($current_config_json->{AmbientSensor}->{hwmon_device_name});
	my $input = $current_config_json->{AmbientSensor}->{hwmon_input};

	if(length($device_file) < 1) {
		print "Файл не найден для устройства с именем '".$current_config_json->{AmbientSensor}->{hwmon_device_name}."', проверьте правильность ввода имени устройства в конфигурационном файле, а так же наличие устройств с таким именем в разделе <a href=\"$prog?action=system_device_diagnostics\">Диагностика Устройств</a>\n";
		footer();
		exit(0);
	}

	$device_file =~ s/\/name/\/$input/g;

	while(1) {
		$! = 0;
		$ret = $s->can_read(.5);
		if($ret != 0) {
			last;
		}

		$mvolts = read_file($device_file);
		chop($mvolts);

		print time() . " - millivolts: $mvolts<BR>\n";
		print "<script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
		STDOUT->flush();
	}

	print "BYE\n";

	footer();
	exit(0);

}

if($action eq 'handset_state') {

	$| = 1;
	use IO::Select;
	$s = IO::Select->new();
	$s->add(\*STDIN);

	header();
	print "Датчик напряжения в аналоговой линии:\n";
	print "<HR>\n";

	my $device_file = FindHwmonDevice($current_config_json->{HandsetState}->{hwmon_device_name});
	my $input = $current_config_json->{HandsetState}->{hwmon_input};

	if(length($device_file) < 1) {
		print "Файл не найден для устройства с именем '".$current_config_json->{HandsetState}->{hwmon_device_name}."', проверьте правильность ввода имени устройства в конфигурационном файле, а так же наличие устройств с таким именем в разделе <a href=\"$prog?action=system_device_diagnostics\">Диагностика Устройств</a>\n";
		footer();
		exit(0);
	}

	$device_file =~ s/\/name/\/$input/g;

	while(1) {
		$! = 0;
		$ret = $s->can_read(.5);
		if($ret != 0) {
			last;
		}

		$mvolts = read_file($device_file);
		chop($mvolts);

		print time() . " - millivolts: $mvolts<BR>\n";
		print "<script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
		STDOUT->flush();
	}

	print "BYE\n";

	footer();
	exit(0);

}


if($action eq 'doors_and_buttons_sensors') {

	$| = 1;
	use IO::Select;

	header();
	print "Ожидание сработки датчика дверей или нажатия кнопок:\n";
	print "<HR>\n";
	print "<PRE>\n";

	my $dev = FindInputDevice($current_config_json->{DoorsAndButtons}->{input_device_name});

	if(length($dev) < 1) {
		print "Файл не найден для устройства с именем '".$current_config_json->{DoorsAndButtons}->{input_device_name}."', проверьте правильность ввода имени устройства в конфигурационном файле, а так же наличие устройств с таким именем в разделе <a href=\"$prog?action=system_device_diagnostics\">Диагностика Устройств</a>\n";
		footer();
		exit(0);
	}

	printf(STDERR "Opening device: $dev\n");

	my $fd_event = \*EVENT;
	my $fd_stdin = \*STDIN;

	my $pid = open($fd_event, "/usr/bin/evtest ".$dev."|");

	$s = IO::Select->new();
	$s->add($fd_stdin);
	$s->add($fd_event);

	my $terminate = 0;

	while($terminate == 0) {

		@fds = $s->can_read(1);

		foreach my $fd (@fds) {
			if($fd == $fd_stdin) { $terminate = 1; next; }

			$msg = <$fd>;
			chop($msg);

			if($msg =~ /EV_KEY/) {
				$msg =~ s/code ([0-9]*)/code <font color=red><B>\1<\/B><\/font>/g;
			}

			print time() . ": $msg "."<script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
		}

		@fds = $s->has_exception(0);

		foreach my $fd (@fds) {
			$terminate = 1;
		}
	}

	kill(9, $pid);
	close($fd_event);

	footer();
	exit(0);
}


if($action eq 'system_view_monitor') {

	$filter = $query->param("filter");

	$host = $current_config_json->{SunBus}->{ManagmentPort}->{ip};
	$port = $current_config_json->{SunBus}->{ManagmentPort}->{port};
	$prefix = $current_config_json->{SunBus}->{SpyLinePrefix};

	if($host eq "0.0.0.0") { $host = "127.0.0.1"; }

	header();

	print "Монитор системных событий: ($host:$port,$prefix)<BR>\n";
	print "<FORM method=\"GET\" action=\"$prog\">Фильтр событий: <INPUT name=filter width=100 value=\"$filter\"><INPUT type=submit value=\"Применить\"><INPUT type=hidden name=action value=system_view_monitor></FORM>\n";

	print<<END;
<script>


function eventPage (sURL) {
  var oHiddFrame = document.createElement("iframe");
  //oHiddFrame.onload = setPrint;
  //oHiddFrame.style.visibility = "hidden";
  oHiddFrame.style.position = "fixed";
  oHiddFrame.style.left = "0";
  oHiddFrame.style.width = "100%";
  oHiddFrame.style.bottom = "0";
  oHiddFrame.style.height = "85%";
  oHiddFrame.src = sURL;
  document.body.appendChild(oHiddFrame);
}

eventPage("$prog?action=system_view_monitor_events&filter=$filter");
</script>
END

	footer();
	exit(0);
}


if($action eq 'system_test_call_init') {

	$apt_num = $query->param("apt_num");
	$filter = $query->param("filter");

	my $call_id = "WEBADMIN_INITIATED_CALL_".int(rand(1000));
	my $call_cmd = "msg: user TAG:GUI_CMD TEXT:{command:\"call\", apt_num:\"$apt_num\", id:\"$call_id\"}";
	my $cancel_cmd = "msg: user TAG:GUI_CMD TEXT:{command:\"disconnect\", apt_num:\"$apt_num\", id:\"$call_id\"}";
	my $call_cmd_escaped = uri_escape_utf8($call_cmd);
	my $cancel_cmd_escaped = uri_escape_utf8($cancel_cmd);

	$host = $current_config_json->{SunBus}->{ManagmentPort}->{ip};
	$port = $current_config_json->{SunBus}->{ManagmentPort}->{port};
	$prefix = $current_config_json->{SunBus}->{SpyLinePrefix};

	if($host eq "0.0.0.0") { $host = "127.0.0.1"; }

	header();

	print "<FORM method=\"GET\" action=\"$prog\"><INPUT name=cmd type=hidden value=\"$cancel_cmd_escaped\"><INPUT type=submit value=\"Отменить вызов на $apt_num\"><BR><INPUT type=hidden name=action value=system_view_monitor_events>\n";
	print "Идентификатор звонка: <B>$call_id</B><BR>Команда: <B>$call_cmd</B></FORM>\n";


	print<<END;
<script>


function eventPage (sURL) {
  var oHiddFrame = document.createElement("iframe");
  //oHiddFrame.onload = setPrint;
  //oHiddFrame.style.visibility = "hidden";
  oHiddFrame.style.position = "fixed";
  oHiddFrame.style.left = "0";
  oHiddFrame.style.width = "100%";
  oHiddFrame.style.bottom = "0";
  oHiddFrame.style.height = "82%";
  oHiddFrame.src = sURL;
  document.body.appendChild(oHiddFrame);
}

eventPage("$prog?action=system_view_monitor_events&filter=$filter&cmd=$call_cmd_escaped");
</script>
END

	print "<SCRIPT>alert('Инициирован звонок на помещение: $apt_num');</SCRIPT>\n";

	exit(0);
}


if($action eq 'system_view_monitor_events') {

	my $filter = $query->param("filter");
	if(length($filter) < 1) { $filter = '.*';}

	my $cmd_escaped = $query->param("cmd");
	my $cmd = uri_unescape($cmd_escaped);

	$| = 1;
	use IO::Select;
	use Socket;

	$host = $current_config_json->{SunBus}->{ManagmentPort}->{ip};
	$port = $current_config_json->{SunBus}->{ManagmentPort}->{port};
	$prefix = $current_config_json->{SunBus}->{SpyLinePrefix};

	if($host eq "0.0.0.0") { $host = "127.0.0.1"; }

	header_simple();

	print "<BODY style=\"background-color:black;color:white\">\n";
	print "<PRE>\n";


	my $fd_socket = \*SOCKET;
	my $fd_stdin = \*STDIN;

	socket($fd_socket, PF_INET,SOCK_STREAM,(getprotobyname('tcp'))[2]);

	if(!connect( $fd_socket, pack_sockaddr_in($port, inet_aton($host)))) {
		print "Ошибка соединения с системным монитором (host:$host, port:$port), проверьте правильность параметров в секции SunBus/ManagmentPort.\n";
		footer();
		close($fd_socket);
		exit(0);
	}

	my $msg = "$prefix core TEXT:ReadyForJob\n";

	syswrite($fd_socket, $msg, length($msg));

	syswrite($fd_socket, $cmd."\n", length($cmd) + 1) if(length($cmd) > 0);


	$s = IO::Select->new();
	$s->add($fd_stdin);
	$s->add($fd_socket);

	my $terminate = 0;

	while($terminate == 0) {

		@fds = $s->can_read(1);

		foreach my $fd (@fds) {

			if($fd == $fd_stdin) { $terminate = 1; next; }

			$msg = <$fd>;

			$msg =~ s/\</&lt;/g;
			$msg =~ s/\>/&gt;/g;

			chop($msg);

			if($msg =~ $filter) {

				my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
				my $ts = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min, $sec);

				if($msg =~ /:AudioStreamer:/) { $msg = "<font color=blue>$msg</font>"; }
				elsif($msg =~ /:VideoStreamer:/) { $msg = "<font color=green>$msg</font>"; }
				elsif($msg =~ /:SuperGlue:/) { $msg = "<font color=yellow>$msg</font>"; }
				elsif($msg =~ /:SIP:/) { $msg = "<font color=magenta>$msg</font>"; }
				elsif($msg =~ /:GUI:/) { $msg = "<font color=pink>$msg</font>"; }
				else { $msg = "<font color=gray>$msg</font>"; }
				print "$ts $msg "."<script> window.scrollTo({ left: 0, top: document.body.scrollHeight, behavior: 'smooth' }); </script>\n";
				#print time() . " $msg "."<script> window.scrollTo({ left: 0, top: document.body.scrollHeight}); </script>\n";
				#print time() . " $msg \n";
				#STDOUT->flush();
			}
		}

		@fds = $s->has_exception(0);

		foreach my $fd (@fds) {
			$terminate = 1;
		}
	}

	print "BYE\n";

	close($fd_socket);

	footer();
	exit(0);

}


if($action eq 'system_view_modules_version') {

	header();

	print "Версии основных модулей ПО:\n";
	print "<HR>\n";
	print "<CENTER><TABLE border=1>\n";
	print "<TR align=center><TD> <B>Имя модуля</B> </TD><TD> <B>Дата/время компиляции</B> </TD><TD> <B>Версия</B> </TD> <TD> <B>Сообщение об ошибке от модуля</B> </TD></TR>\n";
	open(VER, $sunbus_version_file);
	while(<VER>) {
		chop;
		$line = $_;
		($module, $_) = split (' ', $_);
		if($line =~ /\"(.*)\" \"(.*)\"/g) {
			$date_version = $1;
			$last_error = $2;
		} else {
			$date_version = "";
			$last_error = "";
		}
		($date_time, $version) = split(' - ', $date_version);
		print "<TR align=center><TD> $module </TD><TD> $date_time </TD><TD> $version </TD><TD> $last_error </TD></TR>\n";
	}
	close(VER);
	print "</TABLE>\n";

	footer();
	exit(0);
}

if($action eq 'system_upload_update') {

	header();

	my $media_dir = $current_config_json->{HTTPRequester}->{media_dir};

	print "Спул обновлений:\n";
	print "<HR>\n";
	print "<FORM method=\"POST\" action=\"$prog\" enctype='multipart/form-data'>\n";
	print "<INPUT type=hidden name=action value=system_upload_update_submit>\n";
	print "<TABLE>\n";
	print "<TR><TD><INPUT name=file type=file accept='application/tar+gzip'></TD><TD><INPUT type=submit value='Загрузить' style='width:100%'></TD></TR>\n";
	print "</TABLE>\n";
	print "</FORM>\n";
	print "<HR>\n";
	print "Файлы помещенные в этот спул будут проинсталлированы и удалены при следующей перезагрузке системы!\n";
	print "<P>\n";
	print "<P>\n";
	print "Загруженные файлы:\n";
	print "<HR>\n";
	print "<FORM><PRE>\n";
	open(LS, "/bin/ls -l ". $media_dir."/firmware_updates/ |");
	while(<LS>) {
		chop;
		($a1, $a2, $a3, $a4, $a5, $a6, $a7, $a8, $a9, $a10) = split (' ', $_);
		if(length($a9) < 1) { next; }
		print "<INPUT type=button value=\"X\" onClick=\"if(confirm('Удалить файл: $a9 ?')){self.location='$prog?action=system_delete_update_file&name=$a9'}\"> $_ \n";
	}
	close(LS);
	print "</PRE></FORM>\n";

	footer();
	exit(0);
}


if($action eq 'system_upload_update_submit') {

	$file_handle = $query->upload("file");
	$file_name = $query->param("file");


	my $total_bytes = 0;
	my $media_dir = $current_config_json->{HTTPRequester}->{media_dir};
	my $store_file_name = $media_dir."/firmware_updates/".$file_name;

	header();

	if(!open (OUTFILE, '>', $store_file_name)) {
		
		print "<script>alert(\"Невозможно сохранить файл $file_name. Ошибка: $!\")</script>\n";

		footer();
		exit(0);
	}

	binmode STDIN;
	binmode OUTFILE;

	while ($bytesread = sysread($file_handle, $buffer, 1024)) {
		print OUTFILE $buffer;
		$total_bytes += $bytesread;
		##printf(STDERR "READ: $bytesread\n");
	}

	print "<script>alert(\"Файл $file_name загружен и сохранен в спуле для обновлений. Принято $total_bytes байт.\")</script>\n";
	print "<script>self.location='$prog?action=system_upload_update';</script>\n";

	footer();
	exit(0);
}


if($action eq 'system_delete_update_file') {

	$name = $query->param("name");
	$name =~ s/\.\.\///g;

	my $media_dir = $current_config_json->{HTTPRequester}->{media_dir};
	my $file = $media_dir."/firmware_updates/".$name;

	header();
	
	if(-f $file) {
		unlink $file;
		print "<script>alert(\"Файл $name удален!\");self.location=\"$prog?action=system_upload_update\";</script>\n";
	} else {
		print "<script>alert(\"Файл $name не найден!\");self.location=\"$prog?action=system_upload_update\";</script>\n";
	}

	footer();
	exit(0);
}



sub save_config {
	$serial = time();
	$current_config_json->{Config}->{serial} = $serial; 
	$current_config_json->{Config}->{source} = 'WebAdmin - '.$ENV{REMOTE_ADDR}; 

	$config_file = $ENV{X_CONFIG_SPOOL_DIR}. "/" . $ENV{X_CONFIG_FILE_NAME} . "." . $serial;
	$config_file_md5 = $config_file . ".md5";
	$config_file_symlink = $ENV{X_CONFIG_SPOOL_DIR}. "/" . $ENV{X_CONFIG_FILE_NAME};

	$data = $json_parser->utf8->encode($current_config_json);

	#write_file($config_file, {binmode => ':utf8'}, $data);
	return undef if(write_file($config_file, $data) == undef);

	$digest = md5_hex($data)."\n";

	return undef if(write_file($config_file_md5, {binmode => ':utf8'}, $digest) == undef);

	unlink($config_file_symlink);

	return symlink($config_file, $config_file_symlink);

	#print "<pre>\n";
	#print Dumper($current_config_json)."\n";
	#print "</pre>\n";
}


sub header_simple {

	print "Content-type: text/html; charset=utf-8\n\n";

	print "<head>\n";
	print "<meta charset=\"utf-8\" />\n";
	print "<meta http-equiv=\"cache-control\" content=\"no-cache\"/>\n";
	print "<meta http-equiv=\"pragma\" content=\"no-cache\"/>\n";
	print "</head>\n";
}

sub header_simple_with_style {

	print "Content-type: text/html; charset=utf-8\n\n";

	print "<head>\n";
	print "<meta charset=\"utf-8\" />\n";
	print "<meta http-equiv=\"cache-control\" content=\"no-cache\"/>\n";
	print "<meta http-equiv=\"pragma\" content=\"no-cache\"/>\n";
	print "<link rel='stylesheet' id='theme_stylesheet' href='bootstrap-combined.min.css' -->\n";
	print "<link rel='stylesheet' id='icon_stylesheet' href='font-awesome.css'>\n";
	print "</head>\n";
}

sub header {

	print "Content-type: text/html; charset=utf-8\n\n";

	print "<head>\n";
	print "<meta charset=\"utf-8\" />\n";
	print "<meta http-equiv=\"cache-control\" content=\"no-cache\"/>\n";
	print "<meta http-equiv=\"pragma\" content=\"no-cache\"/>\n";
	print "<link rel='stylesheet' id='theme_stylesheet' href='bootstrap-combined.min.css' -->\n";
	print "<link rel='stylesheet' id='icon_stylesheet' href='font-awesome.css'>\n";
	print "</head>\n";
	print "<body>\n";

	print "<div align=right>";
	print "[<a href=\"$prog?action=list_sections\">Список конфигурационных разделов</a>] ";
	print "[<a href=\"$prog?action=network_settings\">Сетевые настройки</a>] ";
	print "[<a href=\"$prog?action=system_settings\">Системные операции</a>] ";
	print "[<a href=\"$prog?action=sensors\">Показания датчиков</a>] ";
	print "</div>\n";
}


sub footer {
	print "\n<HR>\n";
	print "<center>{SID: <b>".$ENV{X_SID}."</b>} {Serial: <b>".$loaded_config_serial."</b>} {Source: ".$loaded_config_source."}</center>\n";
	print "</body>\n";
}


sub FindHwmonDevice {
	my $device_name = shift @_;
	my $i;
	
	for($i = 0; $i < 10; $i++) {
		my $tmp_str;
		$tmp_str = sprintf("/sys/class/hwmon/hwmon%d/name", $i);

		if(-f $tmp_str) {

			$name = read_file($tmp_str);
			chop $name;

			#printf(STDERR "Device name: %s\n", name);

			if($name eq $device_name) {
				return $tmp_str;
			}
		}
	}

	return "";
}


sub FindInputDevice {
	my $device_name = shift @_;
	my $i;
	
	for($i = 0; $i < 10; $i++) {
		my $tmp_str;
		$tmp_str = sprintf("/dev/input/event%d", $i);

		if(-e $tmp_str) {

			open(my $EVDEVNAME, "$evdevname $tmp_str|") or next;

			$name = <$EVDEVNAME>;
			chop $name;
			close($EVDEVNAME);

			if($name eq $device_name) {
				return $tmp_str;
			}
		}
	}

	return "";
}



print "Content-type: text/plain; charset=utf-8\n\n";

print "Error occured. action = $action\n\n";

print Dumper($query)."\n\n"; 

system("env");

exit(1);


sub uri_escape {
    my($text, $patn) = @_;
    return undef unless defined $text;
    if (defined $patn){
	unless (exists  $subst{$patn}) {
	    # Because we can't compile the regex we fake it with a cached sub
	    (my $tmp = $patn) =~ s,/,\\/,g;
	    eval "\$subst{\$patn} = sub {\$_[0] =~ s/([$tmp])/\$escapes{\$1} || _fail_hi(\$1)/ge; }";
	    printf(STDERR "uri_escape: $@") if $@;
	}
	&{$subst{$patn}}($text);
    } else {
	$text =~ s/($Unsafe{RFC3986})/$escapes{$1} || _fail_hi($1)/ge;
    }
    $text;
}

sub _fail_hi {
    my $chr = shift;
    printf(STDERR "Can't escape \\x{%04X}, try uri_escape_utf8() instead", ord($chr));
}

sub uri_escape_utf8 {
    my $text = shift;
    return undef unless defined $text;
    utf8::encode($text);
    return uri_escape($text, @_);
}

sub uri_unescape {
    # Note from RFC1630:  "Sequences which start with a percent sign
    # but are not followed by two hexadecimal characters are reserved
    # for future extension"
    my $str = shift;
    if (@_ && wantarray) {
	# not executed for the common case of a single argument
	my @str = ($str, @_);  # need to copy
	for (@str) {
	    s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
	}
	return @str;
    }
    $str =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg if defined $str;
    $str;
}

# XXX FIXME escape_char is buggy as it assigns meaning to the string's storage format.
sub escape_char {
    # Old versions of utf8::is_utf8() didn't properly handle magical vars (e.g. $1).
    # The following forces a fetch to occur beforehand.
    my $dummy = substr($_[0], 0, 0);

    if (utf8::is_utf8($_[0])) {
	my $s = shift;
	utf8::encode($s);
	unshift(@_, $s);
    }

    return join '', @escapes{split //, $_[0]};
}

